/* Classic Ladder Project */
/* Copyright (C) 2001 Marc Le Douarain */
/* mavati@club-internet.fr */
/* http://www.multimania.com/mavati/classicladder */
/* February 2001 */
/* Last update : 19 April. 2001 */
/* ---------------------------- */
/* GTK Interface & Main */
/* Inspired from the scribble example.
/* ---------------------------------- */
/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <gtk/gtk.h>

#include "stdio.h"
#include "time.h"

#include "classicladder.h"

StrRung RungArray[NBR_RUNGS];
int FirstRung;
int CurrentRung;
int LastRung;
char VarArray[NBR_BITS];
StrTimer TimerArray[NBR_TIMERS];
StrMonostable MonostableArray[NBR_MONOSTABLES];

static GdkPixmap *pixmap = NULL;
GtkWidget *drawing_area;
GtkWidget *chkvar[40],*entrylabel,*entrycomment;

#include "drawing.c"
#include "vars_access.c"
#include "calc.c"
#include "files.c"


/* Create a new backing pixmap of the appropriate size */
static gint configure_event( GtkWidget         *widget,
                            GdkEventConfigure *event )
{
    if (pixmap)
        gdk_pixmap_unref(pixmap);

    pixmap = gdk_pixmap_new(widget->window,
                            widget->allocation.width,
                            widget->allocation.height,
                            -1);
    gdk_draw_rectangle (pixmap,
                        widget->style->white_gc,
                        TRUE,
                        0, 0,
                        widget->allocation.width,
                        widget->allocation.height);
    return TRUE;
}

/* Redraw the screen from the backing pixmap */
static gint expose_event( GtkWidget      *widget,
                        GdkEventExpose *event )
{
    gdk_draw_pixmap(widget->window,
                    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                    pixmap,
                    event->area.x, event->area.y,
                    event->area.x, event->area.y,
                    event->area.width, event->area.height);

    return FALSE;
}

/* Draw a rectangle on the screen */
/*static void draw_brush( GtkWidget *widget,
                        gdouble    x,
                        gdouble    y)
{
    GdkRectangle update_rect;

    update_rect.x = x - 5;
    update_rect.y = y - 5;
    update_rect.width = 100;
    update_rect.height = 100;
    gdk_draw_rectangle (pixmap,
                        widget->style->black_gc,
                        TRUE,
                        update_rect.x, update_rect.y,
                        update_rect.width, update_rect.height);
    gtk_widget_draw (widget, &update_rect);
}*/

static gint button_press_event( GtkWidget      *widget,
                                GdkEventButton *event )
{
// future : making the ladder editor  ! ;-)
//    if (event->button == 1 && pixmap != NULL)
//        draw_brush (widget, event->x, event->y);
    return TRUE;
}

void quit ()
{
    gtk_exit (0);
}

static gint chkvar_press_event( GtkWidget      *widget, char * var )
{
    if (gtk_toggle_button_get_active((GtkToggleButton *)widget))
        *var = 1;
    else
        *var = 0;
    return TRUE;
}

static gint cyclic_refresh(gpointer data)
{
    RefreshAllRungs();
    DrawRung(&RungArray[CurrentRung],FALSE);
}

void refresh_label_comment(StrRung * RfhRung)
{
    gtk_entry_set_text((GtkEntry *)entrylabel,RfhRung->Label);
    gtk_entry_set_text((GtkEntry *)entrycomment,RfhRung->Comment);
}

void buttonprev_click()
{
    if (CurrentRung!=FirstRung)
    {
        CurrentRung = RungArray[CurrentRung].PrevRung;
        refresh_label_comment(&RungArray[CurrentRung]);
        DrawRung(&RungArray[CurrentRung],TRUE);
    }
}
void buttonnext_click()
{
    if (CurrentRung!=LastRung)
    {
        CurrentRung = RungArray[CurrentRung].NextRung;
        refresh_label_comment(&RungArray[CurrentRung]);
        DrawRung(&RungArray[CurrentRung],TRUE);
    }
}

int main( int   argc, char *argv[] )
{
    GtkWidget *window,*windowvars;
    GtkWidget *vbox,*hboxtop,*vboxvars,*vboxvars2,*hboxvars;

    GtkWidget *buttonquit,*buttonprev,*buttonnext;
    int NumVar,ColumnVar;

    gtk_init (&argc, &argv);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title ((GtkWindow *)window, "Rung Display");

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
                        GTK_SIGNAL_FUNC (quit), NULL);
    hboxtop = gtk_hbox_new (FALSE,0);
    gtk_container_add (GTK_CONTAINER (vbox), hboxtop);
    gtk_widget_show(hboxtop);

    entrylabel = gtk_entry_new();
    gtk_widget_set_usize((GtkWidget *)entrylabel,80,0);
    gtk_entry_set_max_length((GtkEntry *)entrylabel,LGT_LABEL);
    gtk_entry_prepend_text((GtkEntry *)entrylabel,"");
    gtk_box_pack_start (GTK_BOX (hboxtop), entrylabel, FALSE, FALSE, 0);
    gtk_widget_show(entrylabel);
    entrycomment = gtk_entry_new();
    gtk_entry_set_max_length((GtkEntry *)entrycomment,LGT_COMMENT);
    gtk_entry_prepend_text((GtkEntry *)entrylabel,"");
    gtk_box_pack_start (GTK_BOX (hboxtop), entrycomment, FALSE, FALSE, 0);
    gtk_widget_show(entrycomment);

    buttonprev = gtk_button_new_with_label ("Prev");
    gtk_box_pack_start (GTK_BOX (hboxtop), buttonprev, FALSE, FALSE, 0);
    gtk_signal_connect(GTK_OBJECT (buttonprev), "clicked",
                        (GtkSignalFunc) buttonprev_click, 0);
    gtk_widget_show (buttonprev);
    buttonnext = gtk_button_new_with_label ("Next");
    gtk_box_pack_start (GTK_BOX (hboxtop), buttonnext, FALSE, FALSE, 0);
    gtk_signal_connect(GTK_OBJECT (buttonnext), "clicked",
                        (GtkSignalFunc) buttonnext_click, 0);
    gtk_widget_show (buttonnext);

    /* Create the drawing area */
    drawing_area = gtk_drawing_area_new ();
    gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area) ,
                            BLOCK_WIDTH*RUNG_WIDTH+OFFSET_X+5 ,
                            BLOCK_HEIGHT*RUNG_HEIGHT+OFFSET_Y);
    gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
    gtk_widget_show (drawing_area);

    buttonquit = gtk_button_new_with_label ("Quit");
    gtk_box_pack_start (GTK_BOX (vbox), buttonquit, FALSE, FALSE, 0);
    gtk_signal_connect_object (GTK_OBJECT (buttonquit), "clicked",
                                GTK_SIGNAL_FUNC (gtk_widget_destroy),
                                GTK_OBJECT (window));
    gtk_widget_show (buttonquit);


    /* Signals used to handle backing pixmap */
    gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
                        (GtkSignalFunc) expose_event, NULL);
    gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",
                        (GtkSignalFunc) configure_event, NULL);

    /* Event signals */
    gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
                        (GtkSignalFunc) button_press_event, NULL);

    gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
                            | GDK_LEAVE_NOTIFY_MASK
                            | GDK_BUTTON_PRESS_MASK
                            | GDK_POINTER_MOTION_MASK
                            | GDK_POINTER_MOTION_HINT_MASK);

    windowvars = gtk_window_new (GTK_WINDOW_TOPLEVEL);
//gtk_window_set_default_size((GtkWindow *)windowvars,100,500);
    gtk_window_set_title ((GtkWindow *)windowvars, "Vars");
    hboxvars = gtk_hbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (windowvars), hboxvars);
    gtk_widget_show (hboxvars);
    vboxvars = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (hboxvars), vboxvars);
    vboxvars2 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (hboxvars), vboxvars2);
    gtk_widget_show (vboxvars);
    gtk_widget_show (vboxvars2);

    /* 2x20 Vars Bits */
    NumVar = 0;
    for(ColumnVar=0; ColumnVar<2; ColumnVar++)
    {
        int OffVar;
        for(OffVar=0; OffVar<20; OffVar++)
        {
            char BufNumVar[20];
            sprintf(BufNumVar,"Bit %d",NumVar);
            chkvar[NumVar] = gtk_check_button_new_with_label(BufNumVar);
            gtk_box_pack_start ((ColumnVar==0)?GTK_BOX (vboxvars):GTK_BOX (vboxvars2), chkvar[NumVar], TRUE, TRUE, 0);
            gtk_widget_show(chkvar[NumVar]);
            gtk_signal_connect(GTK_OBJECT (chkvar[NumVar]), "toggled",
                                (GtkSignalFunc) chkvar_press_event, &VarArray[NumVar]);
            NumVar++;
        }
    }

    gtk_widget_show (window);
    gtk_widget_show (windowvars);

    InitVars();
    LoadTimersParams("timers.csv",TimerArray);
    LoadMonostablesParams("monostables.csv",MonostableArray);
    PrepareTimers();
    PrepareMonostables();
    InitRungs();
    LoadAllRungs("rung_example_",RungArray,&FirstRung,&LastRung,&CurrentRung);
//    DumpRung(&RungArray[0]);
    PrepareRungs();

    refresh_label_comment(&RungArray[CurrentRung]);
    gtk_timeout_add(TIME_REFRESH_RUNG_MS,cyclic_refresh,NULL);

    gtk_main();

    return 0;
}

